#define DEBUG

#include <GPNBadge.hpp>
#include "BadgeUI.h"
#include "LaserMenuItem.hpp"
#include "Player.hpp"
#include "url-encode.h"
#include "PlayerListMenu.hpp"
#include "GameClient.hpp"
#include "GameServer.hpp"
#include "rboot.h"
#include "rboot-api.h"

#define USEIR

#ifdef DEBUG
#define BADGE_PULL_INTERVAL 10000
#else
#define BADGE_PULL_INTERVAL 5*60*1000
#endif

// Color definitions
#define BLACK   0x0000
#define BLUE    0x001F
#define RED     0xF800
#define GREEN   0x07E0
#define CYAN    0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE   0xFFFF

void hostGame();

void joinGame();


Badge badge;
WindowSystem *ui = new WindowSystem(&tft);
Menu *role_menu = new Menu(2); // 2 Items per page
Player *player;
GameServer *server;
GameClient *client;
char pixIndex = 0;
bool changePixels = true;
unsigned long  lastChange = 0;
unsigned long lastNotificationPull = 0;

void setup() {
    badge.init();
    char *nickname = loadNick();
    player = new Player(&badge, nickname);
    badge.setBacklight(true);
    badge.setGPIO(IR_EN, LOW);
    badge.setAnalogMUX(MUX_JOY);

    ui->setTheme(new ThemeDark());
    ui->open(role_menu);

    role_menu->addMenuItem(new MenuItem("Host", hostGame));
    role_menu->addMenuItem(new MenuItem("Join", joinGame));
    ui->draw();
    rboot_config rboot_config = rboot_get_config();
    SPIFFS.begin();
    File f = SPIFFS.open("/rom" + String(rboot_config.current_rom), "w");
    f.println("Lasertag\n");
    connectBadge();
}

void hostGame() {
#ifdef DEBUG
    Serial.printf("Hostotter\n");
#endif
    server = new GameServer(ui, &badge);
#ifdef DEBUG
    Serial.printf("Host otter nach init\n");
#endif

    client = new GameClient(ui, &badge, player);
#ifdef DEBUG
    Serial.printf("Host - Ende\n");
#endif
}

String recv_ir() {
    badge.setGPIO(IR_EN, HIGH);
    Serial.println("Entering receive mode");
    String received = "";
    bool stringCompleted = false;
    bool dataCompleted = false;
    bool dataVerified = false;
    uint8_t checksumRec = 0;

    //bool done = false;

    while (!dataCompleted) {
        int on_time = 0;
        decode_results results;        // Somewhere to store the results
        if (irrecv.decode(&results)) {
            if (results.overflow) {
                Serial.println("IR code too long. Edit IRremoteInt.h and increase RAWBUF");
                return "a12\n7.0\n.0.\n1b";
            }
            char *buf = reinterpret_cast<char *>(&results.value);

            if (millis() - on_time > 500) {
                pixels.setPixelColor(1, pixels.Color(0, 0, 0));
                pixels.show();
            }
            if (stringCompleted) {
                uint8_t checksum = buf[1];
                dataCompleted = true;

                Serial.printf("checksum?? %c;%c\n", checksumRec << 8 | 222, checksum);

                if (checksum == checksumRec) {
                    dataVerified = true;
                    pixels.setPixelColor(1, pixels.Color(0, 100, 0));
                } else {
                    pixels.setPixelColor(1, pixels.Color(100, 0, 0));
                }
                pixels.show();
                on_time = millis();
            } else {
                received += String(buf[0]) + buf[1] + buf[2] + buf[3];
                checksumRec += buf[0] + buf[1] + buf[2] + buf[3];
                Serial.printf("->:\n\t0: %c\n\t1: %c\n\t2: %c\n\t3: %c\n", buf[0], buf[1], buf[2], buf[3]);
                if (buf[0] == '\n' || buf[1] == '\n' || buf[2] == '\n' || buf[3] == '\n') {

                    stringCompleted = true;
                    Serial.printf("string complete %s\n", received.c_str());
//done = true; //isValidIp(received.substring(1, received.length() - 2));
                }
            }
            irrecv.resume();              // Prepare for the next value
        }
        delay(0);
        //
    }
    badge.setGPIO(IR_EN, LOW);
    return received;
}

void joinGame() {
    ui->closeCurrent();
    ui->draw();

#ifdef DEBUG
    Serial.printf("Join Otter\n");
#endif

    String irinput = recv_ir();
    String ip;
    ip = irinput.substring(1, irinput.length() - 1);
    unsigned short firstA = ip.indexOf('a');
    Serial.printf("firstA: %d\n", firstA);
    ip = ip.substring(firstA, ip.length());
    Serial.printf("Received ip: >%s<\n", ip.c_str());

    client = new GameClient(ui, &badge, player);
    client->joinGame(ip);
#ifdef DEBUG
    Serial.printf("Join - Ende\n");
#endif

}

void loop() {
    ui->dispatchInput(badge.getJoystickState());
    ui->draw();

    if (millis() - lastNotificationPull > BADGE_PULL_INTERVAL) {
        if (connectBadge()) {
            pixels.setPixelColor(pixIndex, pixels.Color(0, 0, 0));
            pixels.setPixelColor(0, pixels.Color(0, 255, 0));
            changePixels = false;
        }
        lastNotificationPull = millis();
    }
    if (changePixels && millis() - lastChange > 1200) {
        pixels.setPixelColor(pixIndex, pixels.Color(0, 0, 0));
        pixIndex = (pixIndex + 1) % 4;
        pixels.setPixelColor(pixIndex, pixels.Color(255, 0, 0));
        lastChange = millis();
    }
    pixels.show();

    if (server) {
        server->update();
    }
    if (client) {
//        client->update();
    }
}

char *loadNick() {
    if (SPIFFS.exists("/nick.conf")) {
        UrlDecode nickUrlDecode("/nick.conf");
        char *nick = nickUrlDecode.getKey("nick");

#ifdef DEBUG
        Serial.printf("loaded nickname %s!\n", nick);
#endif
        return nick;
    }
    return "otter";
}

bool connectBadge() {
    if (WiFi.status() == WL_CONNECTED) {
        return true;
    }
    WiFi.mode(WIFI_STA);
    delay(30);
    Serial.println("Loading config.");
    File wifiConf = SPIFFS.open("/wifi.conf", "r");
    String configString;
    while (wifiConf.available()) {
        configString += char(wifiConf.read());
    }
    wifiConf.close();
    UrlDecode confParse(configString.c_str());
    Serial.println(configString);
    configString = String();
    char *ssid = confParse.getKey("ssid");
    char *pw = confParse.getKey("pw");
    Serial.printf("Connecting to wifi '%s' with password '%s'...\n", ssid, pw);
    unsigned long startTime = millis();
    WiFi.begin(ssid, pw);
    delete[] pw;
    return WiFi.status() == WL_CONNECTED;
}
